// $Rev: 760 $ $Date: 2010-09-26 15:40:10 -0400 (Sun, 26 Sep 2010) $

/** \file geneoutparams.h */

#ifndef GENEOUT_H
#define GENEOUT_H 1

#include <string>
#include <list>
#include <set>
#include <iomanip>
#include <iostream>
#include <fstream>
#include <math.h>
#include <stdlib.h>
#include <algorithm>
#include "debugoutput.h"
#include "nexus.h"
#include "kernelmethod.h"
#include "jkparams.h"
#include "mbparams.h"
#include "sampleparams.h"
#include "multindparams.h"
#include "tempprefix.h"


using namespace::std;

/// This class will read in the <NAME> NEXUS block which will
/// specify all the parameters to run our software to identify
/// gene outliers.
class GeneOutParameters 
{
public: 
    GeneOutParameters();
    GeneOutParameters(string fileName);
    GeneOutParameters(std::istream &in);
    friend std::ostream& operator << (std::ostream &out, const GeneOutParameters &someGeneOutParameters);
    friend std::istream& operator >> (std::istream &in, GeneOutParameters &someGeneOutParameters);
    void processArg(int argc, char **argv);

    list <string> inputFileNames; // For backwards compatibility
    list <string> inputFileNamesGroupOne;
    list <string> inputFileNamesGroupTwo;

    // Parameters for mrbayes with their default values.
    MrBayesParameters MrBayesParam;
    JackknifeParameters JackknifeParam;
    MultIndParameters MultIndParam;
    SampleParameters SampleParam;

    time_t      randGenSeed;
    int         doMB; // By default, do not do mr bayes
    int         doJackknife;      // Default is to do jackknife
    int         doMultInd;
    int         numInd; // Default is 1 individual per species
    int         noTreeCalc; // Do not skip mr bayes calculation.
    int         concatGroups;   // Default is 0, do not concat groups
    int         numGroupOne; // First file is group one and thats all.
    int         statTest; // 0 mean no stat test, 1 means stat test
    int         geneStatTest; // Do the statistical test based on the hyp test on gene trees
    int         numInitCalc; // 
    int         numStatTests;
    int         skipInitCalc; // Dont skip initial calculation for stat test
    double      mySeparation;
    int         doSim; 
    string      simCommand;
                                    // 1 is jk-align, 2 is tree, 3 is no-jk
    int         multSVMSepMethod;   // When doing multiple svm sep test, which method: jk-align, tree, no-jk
                            // 1 is singlesvmsep, 2 is multiplesvmsep
    int         testType;   // What test to run: single svm calc or multiple svm calc.
    string      tempPrefix;
    int         JKStepOne;
    int         indJackknife; // 1 means use randomly selected alignments to jackknife/bootstrap from.
                              // 0 means to use the concatenated alignments. Default is 0.
    int         permuteOrig;  // 1 means permute original alignments and bootstrap/jackknife to the correct size
                              // 0 means not.
    int         allowAnyPermuation; //1 means allow anything. 0 means dissallow new group one to be all from group one.

protected:
    void init ();
};

#endif
